@@ -24,11 +24,11 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include <weston/compositor.h>
+#include <weston.h>
+#include <weston/ivi-layout-export.h>
 #include "ilm_types.h"
 
 #include "ivi-input-server-protocol.h"
-#include "ivi-controller-interface.h"
 
 struct seat_ctx {
     struct input_context *input_ctx;
@@ -60,7 +60,10 @@ struct input_context {
     struct wl_list controller_list;
     struct wl_list surface_list;
     struct weston_compositor *compositor;
-    const struct ivi_controller_interface *ivi_controller_interface;
+    const struct ivi_layout_interface *ivi_layout_interface;
+
+    struct wl_listener surface_created;
+    struct wl_listener surface_destroyed;
 };
 
 static int
@@ -79,8 +82,8 @@ static int
 add_accepted_seat(struct surface_ctx *surface, const char *seat)
 {
     char **added_entry;
-    const struct ivi_controller_interface *interface =
-        surface->input_context->ivi_controller_interface;
+    const struct ivi_layout_interface *interface =
+        surface->input_context->ivi_layout_interface;
     if (get_accepted_seat(surface, seat) >= 0) {
         weston_log("%s: Warning: seat '%s' is already accepted by surface %d\n",
                    __FUNCTION__, seat,
@@ -110,8 +113,8 @@ remove_accepted_seat(struct surface_ctx *surface, const char *seat)
     int i;
     struct wl_array *array = &surface->accepted_devices;
     char **data = array->data;
-    const struct ivi_controller_interface *interface =
-        surface->input_context->ivi_controller_interface;
+    const struct ivi_layout_interface *interface =
+        surface->input_context->ivi_layout_interface;
     if (seat_index < 0) {
         weston_log("%s: Warning: seat '%s' not found for surface %d\n",
                   __FUNCTION__, seat,
@@ -148,6 +151,20 @@ send_input_focus(struct input_context *ctx, t_ilm_surface surface_id,
     }
 }
 
+static void
+pointer_move(struct weston_pointer *pointer, wl_fixed_t x, wl_fixed_t y)
+{
+    struct weston_pointer_motion_event event;
+
+    event.x = x;
+    event.y = y;
+    event.dx = 0;
+    event.dy = 0;
+    event.mask = WESTON_POINTER_MOTION_ABS;
+
+    weston_pointer_move(pointer, &event);
+}
+
 static void
 set_weston_focus(struct input_context *ctx, struct surface_ctx *surface_ctx,
                  ilmInputDevice focus, struct weston_seat *seat, t_ilm_bool enabled)
@@ -160,46 +177,41 @@ set_weston_focus(struct input_context *ctx, struct surface_ctx *surface_ctx,
     wl_fixed_t x, y;
     struct weston_pointer *pointer = weston_seat_get_pointer(seat);
     struct weston_keyboard *keyboard = weston_seat_get_keyboard(seat);
-    const struct ivi_controller_interface *interface =
-              ctx->ivi_controller_interface;
+    const struct ivi_layout_interface *interface =
+              ctx->ivi_layout_interface;
 
     /* Assume one view per surface */
-	w_surface = interface->surface_get_weston_surface(surface_ctx->layout_surface);
+    w_surface = interface->surface_get_weston_surface(surface_ctx->layout_surface);
     view = wl_container_of(w_surface->views.next, view, surface_link);
-	if ((focus & ILM_INPUT_DEVICE_POINTER) &&
-		 (pointer != NULL)){
-		if ( (view != NULL) && enabled) {
-	        weston_view_to_global_fixed(view, wl_fixed_from_int(0),
-	            wl_fixed_from_int(0), &x, &y);
-	        // move pointer to local (0,0) of the view
-	        weston_pointer_move(pointer, x, y);
-		    weston_pointer_set_focus(pointer, view,
-		        wl_fixed_from_int(0), wl_fixed_from_int(0));
-		} else if (pointer->focus == view){
+    if ((focus & ILM_INPUT_DEVICE_POINTER) && (pointer != NULL)){
+        if ( (view != NULL) && enabled) {
+            weston_view_to_global_fixed(view, wl_fixed_from_int(0),
+            wl_fixed_from_int(0), &x, &y);
+            // move pointer to local (0,0) of the view
+            pointer_move(pointer, x, y);
+            weston_pointer_set_focus(pointer, view,
+            wl_fixed_from_int(0), wl_fixed_from_int(0));
+        } else if (pointer->focus == view) {
             weston_pointer_set_focus(pointer, NULL,
-                wl_fixed_from_int(0), wl_fixed_from_int(0));
-		}
-	}
+            wl_fixed_from_int(0), wl_fixed_from_int(0));
+        }
+    }
 
-	if ((focus & ILM_INPUT_DEVICE_KEYBOARD) &&
-		 (keyboard != NULL)) {
-		if (w_surface) {
-	        surface_client = wl_resource_get_client(w_surface->resource);
-	        serial = wl_display_next_serial(ctx->compositor->wl_display);
-	        wl_resource_for_each(resource, &keyboard->resource_list) {
-	            if (wl_resource_get_client(resource) != surface_client)
-	                continue;
+    if ((focus & ILM_INPUT_DEVICE_KEYBOARD) && (keyboard != NULL)) {
+        surface_client = wl_resource_get_client(w_surface->resource);
+        serial = wl_display_next_serial(ctx->compositor->wl_display);
+        resource = wl_resource_find_for_client(&keyboard->resource_list,
+                                               surface_client);
 
-	            if (!enabled) {
-	                wl_keyboard_send_leave(resource, serial,
-	                         w_surface->resource);
-	            } else {
-                    wl_keyboard_send_enter(resource, serial, w_surface->resource,
-                            &keyboard->keys);
-	            }
-	        }
-		}
-	}
+        if (!resource)
+            return;
+
+        if (!enabled)
+            wl_keyboard_send_leave(resource, serial, w_surface->resource);
+        else
+            wl_keyboard_send_enter(resource, serial, w_surface->resource,
+                                   &keyboard->keys);
+    }
 }
 
 static void
@@ -209,8 +221,8 @@ keyboard_grab_key(struct weston_keyboard_grab *grab, uint32_t time,
     struct seat_ctx *seat_ctx = wl_container_of(grab, seat_ctx, keyboard_grab);
     struct surface_ctx *surf_ctx;
     struct wl_display *display = grab->keyboard->seat->compositor->wl_display;
-    const struct ivi_controller_interface *interface =
-        seat_ctx->input_ctx->ivi_controller_interface;
+    const struct ivi_layout_interface *interface =
+        seat_ctx->input_ctx->ivi_layout_interface;
 
     wl_list_for_each(surf_ctx, &seat_ctx->input_ctx->surface_list, link) {
         struct weston_surface *surface;
@@ -227,19 +239,17 @@ keyboard_grab_key(struct weston_keyboard_grab *grab, uint32_t time,
         surface_client = wl_resource_get_client(surface->resource);
         serial = wl_display_next_serial(display);
 
-        wl_resource_for_each(resource, &grab->keyboard->resource_list) {
-            if (wl_resource_get_client(resource) != surface_client)
-                continue;
-
+        resource = wl_resource_find_for_client(&grab->keyboard->resource_list,
+                                               surface_client);
+        if (resource) {
             wl_keyboard_send_key(resource, serial, time, key, state);
+            continue;
         }
 
-        wl_resource_for_each(resource, &grab->keyboard->focus_resource_list) {
-            if (wl_resource_get_client(resource) != surface_client)
-                continue;
-
+        resource = wl_resource_find_for_client(&grab->keyboard->focus_resource_list,
+                                               surface_client);
+        if (resource)
             wl_keyboard_send_key(resource, serial, time, key, state);
-        }
     }
 }
 
@@ -251,8 +261,8 @@ keyboard_grab_modifiers(struct weston_keyboard_grab *grab, uint32_t serial,
     struct seat_ctx *seat_ctx = wl_container_of(grab, seat_ctx, keyboard_grab);
     struct surface_ctx *surf_ctx;
     struct wl_display *display = grab->keyboard->seat->compositor->wl_display;
-    const struct ivi_controller_interface *interface =
-        seat_ctx->input_ctx->ivi_controller_interface;
+    const struct ivi_layout_interface *interface =
+        seat_ctx->input_ctx->ivi_layout_interface;
 
     wl_list_for_each(surf_ctx, &seat_ctx->input_ctx->surface_list, link) {
         struct weston_surface *surface;
@@ -272,21 +282,17 @@ keyboard_grab_modifiers(struct weston_keyboard_grab *grab, uint32_t serial,
         surface_client = wl_resource_get_client(surface->resource);
         serial = wl_display_next_serial(display);
 
-        wl_resource_for_each(resource, &grab->keyboard->resource_list) {
-            if (wl_resource_get_client(resource) != surface_client)
-                continue;
-
+        resource = wl_resource_find_for_client(&grab->keyboard->resource_list,
+                                               surface_client);
+        if (resource)
             wl_keyboard_send_modifiers(resource, serial, mods_depressed,
-			               mods_latched, mods_locked, group);
-        }
-
-        wl_resource_for_each(resource, &grab->keyboard->focus_resource_list) {
-            if (wl_resource_get_client(resource) != surface_client)
-                continue;
+                                       mods_latched, mods_locked, group);
 
+        resource = wl_resource_find_for_client(&grab->keyboard->focus_resource_list,
+                                               surface_client);
+        if (resource)
             wl_keyboard_send_modifiers(resource, serial, mods_depressed,
-			               mods_latched, mods_locked, group);
-        }
+                                       mods_latched, mods_locked, group);
     }
 }
 
@@ -308,55 +314,38 @@ pointer_grab_focus(struct weston_pointer_grab *grab)
 
 static void
 pointer_grab_motion(struct weston_pointer_grab *grab, uint32_t time,
-                    wl_fixed_t x, wl_fixed_t y)
+                    struct weston_pointer_motion_event *event)
 {
-    struct seat_ctx *seat = wl_container_of(grab, seat, pointer_grab);
-    struct surface_ctx *surf_ctx;
-    const struct ivi_controller_interface *interface =
-        seat->input_ctx->ivi_controller_interface;
+    struct weston_pointer *pointer = grab->pointer;
+    struct weston_compositor *compositor = pointer->seat->compositor;
+    struct wl_list *resource_list;
+    struct wl_resource *resource;
+    struct weston_view *picked_view;
+    wl_fixed_t x, y;
+    wl_fixed_t sx, sy;
+    wl_fixed_t old_sx = pointer->sx;
+    wl_fixed_t old_sy = pointer->sy;
 
-    weston_pointer_move(grab->pointer, x, y);
+    weston_pointer_move(pointer, event);
 
-    /* Get coordinates relative to the surface the pointer is in.
-     * This might cause weirdness if there are multiple surfaces
-     * that are accepted by this pointer's seat and have focus */
+    if (pointer->focus) {
+        weston_pointer_motion_to_abs(pointer, event, &x, &y);
+        picked_view = weston_compositor_pick_view(compositor, x, y,
+                                           &sx, &sy);
 
-    /* For each surface_ctx, check for focus and send */
-    wl_list_for_each(surf_ctx, &seat->input_ctx->surface_list, link) {
-        struct weston_surface *surf;
-        struct wl_resource *resource;
-        struct wl_client *surface_client;
-        struct weston_view *view;
-        wl_fixed_t sx, sy;
+       if (picked_view != pointer->focus)
+           return;
 
-        if (!(surf_ctx->focus & ILM_INPUT_DEVICE_POINTER))
-            continue;
+        pointer->sx = sx;
+        pointer->sy = sy;
+    }
 
-        if (get_accepted_seat(surf_ctx, grab->pointer->seat->seat_name) < 0)
-            continue;
-
-        /* Assume one view per surface */
-        surf = interface->surface_get_weston_surface(surf_ctx->layout_surface);
-        view = wl_container_of(surf->views.next, view, surface_link);
-
-        if (view == grab->pointer->focus) {
-
-            /* Do not send motion events for coordinates outside the surface */
-            weston_view_from_global_fixed(view, x, y, &sx, &sy);
-            if ((!pixman_region32_contains_point(&surf->input, wl_fixed_to_int(sx),
-                                                wl_fixed_to_int(sy), NULL))
-                 && (grab->pointer->button_count == 0))
-                continue;
-
-            surface_client = wl_resource_get_client(surf->resource);
-
-            wl_resource_for_each(resource, &grab->pointer->focus_resource_list) {
-                if (wl_resource_get_client(resource) != surface_client)
-                    continue;
-
-                wl_pointer_send_motion(resource, time, sx, sy);
-            }
-            return;
+    if (pointer->focus_client &&
+        (old_sx != pointer->sx || old_sy != pointer->sy)) {
+        resource_list = &pointer->focus_client->pointer_resources;
+        wl_resource_for_each(resource, resource_list) {
+            wl_pointer_send_motion(resource, time,
+                            pointer->sx, pointer->sy);
         }
     }
 }
@@ -372,12 +361,13 @@ pointer_grab_button(struct weston_pointer_grab *grab, uint32_t time,
     struct surface_ctx *surf_ctx;
     wl_fixed_t sx, sy;
     struct weston_view *picked_view, *w_view, *old_focus;
-    struct weston_surface *w_surf;
+    struct weston_surface *w_surf, *send_surf;
+    struct weston_subsurface *sub;
     struct wl_resource *resource;
-    struct wl_client *surface_client;
+    struct wl_list *resource_list = NULL;
     uint32_t serial;
-    const struct ivi_controller_interface *interface =
-        seat->input_ctx->ivi_controller_interface;
+    const struct ivi_layout_interface *interface =
+        seat->input_ctx->ivi_layout_interface;
 
     picked_view = weston_compositor_pick_view(compositor, pointer->x, pointer->y,
                                        &sx, &sy);
@@ -391,12 +381,24 @@ pointer_grab_button(struct weston_pointer_grab *grab, uint32_t time,
         /* search for the picked view in layout surfaces */
         wl_list_for_each(surf_ctx, &seat->input_ctx->surface_list, link) {
             w_surf = interface->surface_get_weston_surface(surf_ctx->layout_surface);
-            w_view = wl_container_of(w_surf->views.next, w_view, surface_link);
+
+            /* Find a focused surface from subsurface list */
+            send_surf = w_surf;
+            if (!wl_list_empty(&w_surf->subsurface_list)) {
+                wl_list_for_each(sub, &w_surf->subsurface_list, parent_link) {
+                    if (sub->surface == picked_view->surface) {
+                        send_surf = sub->surface;
+                        break;
+                    }
+                }
+            }
+
+            w_view = wl_container_of(send_surf->views.next, w_view, surface_link);
 
             if (get_accepted_seat(surf_ctx, grab->pointer->seat->seat_name) < 0)
                 continue;
 
-            if (picked_view->surface == w_surf) {
+            if (picked_view->surface == send_surf) {
                 /* Correct layout surface is found*/
                 surf_ctx->focus |= ILM_INPUT_DEVICE_POINTER;
                 send_input_focus(seat->input_ctx,
@@ -415,20 +417,41 @@ pointer_grab_button(struct weston_pointer_grab *grab, uint32_t time,
         }
     }
 
-    /* Send to surfaces that have pointer focus */
-    if (grab->pointer->focus == picked_view) {
-        surface_client = wl_resource_get_client(grab->pointer->focus->surface->resource);
+    if (pointer->focus_client)
+        resource_list = &pointer->focus_client->pointer_resources;
+    if (resource_list && !wl_list_empty(resource_list)) {
+        resource_list = &pointer->focus_client->pointer_resources;
         serial = wl_display_next_serial(display);
-
-        wl_resource_for_each(resource, &grab->pointer->focus_resource_list) {
-            if (wl_resource_get_client(resource) != surface_client)
-                continue;
-
-            wl_pointer_send_button(resource, serial, time, button, state);
-        }
+        wl_resource_for_each(resource, resource_list)
+            wl_pointer_send_button(resource,
+                           serial,
+                           time,
+                           button,
+                           state);
     }
 }
 
+static void
+pointer_grab_axis(struct weston_pointer_grab *grab,
+                  uint32_t time,
+                  struct weston_pointer_axis_event *event)
+{
+    weston_pointer_send_axis(grab->pointer, time, event);
+}
+
+static void
+pointer_grab_axis_source(struct weston_pointer_grab *grab,
+                          uint32_t source)
+{
+    weston_pointer_send_axis_source(grab->pointer, source);
+}
+
+static void
+pointer_grab_frame(struct weston_pointer_grab *grab)
+{
+    weston_pointer_send_frame(grab->pointer);
+}
+
 static void
 pointer_grab_cancel(struct weston_pointer_grab *grab)
 {
@@ -438,6 +461,9 @@ static struct weston_pointer_grab_interface pointer_grab_interface = {
     pointer_grab_focus,
     pointer_grab_motion,
     pointer_grab_button,
+    pointer_grab_axis,
+    pointer_grab_axis_source,
+    pointer_grab_frame,
     pointer_grab_cancel
 };
 
@@ -449,8 +475,8 @@ touch_grab_down(struct weston_touch_grab *grab, uint32_t time, int touch_id,
     struct wl_display *display = grab->touch->seat->compositor->wl_display;
     struct surface_ctx *surf_ctx;
     wl_fixed_t sx, sy;
-    const struct ivi_controller_interface *interface =
-        seat->input_ctx->ivi_controller_interface;
+    const struct ivi_layout_interface *interface =
+        seat->input_ctx->ivi_layout_interface;
 
     /* if touch device has no focused view, there is nothing to do*/
     if (grab->touch->focus == NULL)
@@ -458,7 +484,8 @@ touch_grab_down(struct weston_touch_grab *grab, uint32_t time, int touch_id,
 
     /* For each surface_ctx, check for focus and send */
     wl_list_for_each(surf_ctx, &seat->input_ctx->surface_list, link) {
-        struct weston_surface *surf;
+        struct weston_surface *surf, *send_surf;
+        struct weston_subsurface *sub;
         struct weston_view *view;
         struct wl_resource *resource;
         struct wl_client *surface_client;
@@ -469,9 +496,20 @@ touch_grab_down(struct weston_touch_grab *grab, uint32_t time, int touch_id,
         if (get_accepted_seat(surf_ctx, grab->touch->seat->seat_name) < 0)
             continue;
 
+        /* Find a focused surface from subsurface list */
+        send_surf = surf;
+        if (!wl_list_empty(&surf->subsurface_list)) {
+            wl_list_for_each(sub, &surf->subsurface_list, parent_link) {
+                if (sub->surface == grab->touch->focus->surface) {
+                    send_surf = sub->surface;
+                    break;
+                }
+            }
+        }
+
         /* Touches set touch focus */
         if (grab->touch->num_tp == 1) {
-            if (surf == grab->touch->focus->surface) {
+            if (send_surf == grab->touch->focus->surface) {
                 surf_ctx->focus |= ILM_INPUT_DEVICE_TOUCH;
                 send_input_focus(seat->input_ctx,
                                  interface->get_id_of_surface(surf_ctx->layout_surface),
@@ -490,28 +528,24 @@ touch_grab_down(struct weston_touch_grab *grab, uint32_t time, int touch_id,
             continue;
 
         /* Assume one view per surface */
-        view = wl_container_of(surf->views.next, view, surface_link);
+        view = wl_container_of(send_surf->views.next, view, surface_link);
         weston_view_from_global_fixed(view, x, y, &sx, &sy);
 
-        surface_client = wl_resource_get_client(surf->resource);
+        surface_client = wl_resource_get_client(send_surf->resource);
         serial = wl_display_next_serial(display);
-        wl_resource_for_each(resource, &grab->touch->resource_list) {
-            if (wl_resource_get_client(resource) != surface_client)
-                continue;
 
-            wl_touch_send_down(resource, serial, time, surf->resource,
+        resource = wl_resource_find_for_client(&grab->touch->resource_list,
+                                               surface_client);
+        if (resource)
+            wl_touch_send_down(resource, serial, time, send_surf->resource,
                                touch_id, sx, sy);
-        }
-        wl_resource_for_each(resource, &grab->touch->focus_resource_list) {
-            if (wl_resource_get_client(resource) != surface_client)
-                continue;
 
-            wl_touch_send_down(resource, serial, time, surf->resource,
+        resource = wl_resource_find_for_client(&grab->touch->focus_resource_list,
+                                               surface_client);
+        if (resource)
+            wl_touch_send_down(resource, serial, time, send_surf->resource,
                                touch_id, sx, sy);
-        }
     }
-
-
 }
 
 static void
@@ -520,12 +554,13 @@ touch_grab_up(struct weston_touch_grab *grab, uint32_t time, int touch_id)
     struct seat_ctx *seat = wl_container_of(grab, seat, touch_grab);
     struct wl_display *display = grab->touch->seat->compositor->wl_display;
     struct surface_ctx *surf_ctx;
-    const struct ivi_controller_interface *interface =
-        seat->input_ctx->ivi_controller_interface;
+    const struct ivi_layout_interface *interface =
+        seat->input_ctx->ivi_layout_interface;
 
     /* For each surface_ctx, check for focus and send */
     wl_list_for_each(surf_ctx, &seat->input_ctx->surface_list, link) {
-        struct weston_surface *surf;
+        struct weston_surface *surf, *send_surf;
+        struct weston_subsurface *sub;
         struct wl_resource *resource;
         struct wl_client *surface_client;
         uint32_t serial;
@@ -537,30 +572,40 @@ touch_grab_up(struct weston_touch_grab *grab, uint32_t time, int touch_id)
             continue;
 
         surf = interface->surface_get_weston_surface(surf_ctx->layout_surface);
-        surface_client = wl_resource_get_client(surf->resource);
         serial = wl_display_next_serial(display);
-        wl_resource_for_each(resource, &grab->touch->resource_list) {
-            if (wl_resource_get_client(resource) != surface_client)
-                continue;
 
-            wl_touch_send_up(resource, serial, time, touch_id);
-        }
-
-        wl_resource_for_each(resource, &grab->touch->focus_resource_list) {
-            if (wl_resource_get_client(resource) != surface_client)
-                continue;
-
-            wl_touch_send_up(resource, serial, time, touch_id);
+        /* Find a focused surface from subsurface list */
+        send_surf = surf;
+        if (!wl_list_empty(&surf->subsurface_list)) {
+            wl_list_for_each(sub, &surf->subsurface_list, parent_link) {
+                if (sub->surface == grab->touch->focus->surface) {
+                    send_surf = sub->surface;
+                    break;
+                }
+            }
         }
 
         /* Touches unset touch focus */
         if (grab->touch->num_tp == 0) {
-            if (surf == grab->touch->focus->surface)
+            if (send_surf == grab->touch->focus->surface)
                 surf_ctx->focus &= ~ILM_INPUT_DEVICE_TOUCH;
                 send_input_focus(seat->input_ctx,
                                  interface->get_id_of_surface(surf_ctx->layout_surface),
                                  ILM_INPUT_DEVICE_TOUCH, ILM_FALSE);
         }
+
+        surface_client = wl_resource_get_client(send_surf->resource);
+
+        resource = wl_resource_find_for_client(&grab->touch->resource_list,
+                                               surface_client);
+        if (resource)
+            wl_touch_send_up(resource, serial, time, touch_id);
+
+        resource = wl_resource_find_for_client(&grab->touch->focus_resource_list,
+                                               surface_client);
+        if (resource)
+            wl_touch_send_up(resource, serial, time, touch_id);
+
     }
 }
 
@@ -571,12 +616,13 @@ touch_grab_motion(struct weston_touch_grab *grab, uint32_t time, int touch_id,
     struct seat_ctx *seat = wl_container_of(grab, seat, touch_grab);
     struct surface_ctx *surf_ctx;
     wl_fixed_t sx, sy;
-    const struct ivi_controller_interface *interface =
-        seat->input_ctx->ivi_controller_interface;
+    const struct ivi_layout_interface *interface =
+        seat->input_ctx->ivi_layout_interface;
 
     /* For each surface_ctx, check for focus and send */
     wl_list_for_each(surf_ctx, &seat->input_ctx->surface_list, link) {
-        struct weston_surface *surf;
+        struct weston_surface *surf, *send_surf;
+        struct weston_subsurface *sub;
         struct weston_view *view;
         struct wl_resource *resource;
         struct wl_client *surface_client;
@@ -592,20 +638,27 @@ touch_grab_motion(struct weston_touch_grab *grab, uint32_t time, int touch_id,
         view = wl_container_of(surf->views.next, view, surface_link);
         weston_view_from_global_fixed(view, x, y, &sx, &sy);
 
-        surface_client = wl_resource_get_client(surf->resource);
-        wl_resource_for_each(resource, &grab->touch->resource_list) {
-            if (wl_resource_get_client(resource) != surface_client)
-                continue;
-
-            wl_touch_send_motion(resource, time, touch_id, sx, sy);
+        /* Find a focused surface from subsurface list */
+        send_surf = surf;
+        if (!wl_list_empty(&surf->subsurface_list)) {
+            wl_list_for_each(sub, &surf->subsurface_list, parent_link) {
+                if (sub->surface == grab->touch->focus->surface) {
+                    send_surf = sub->surface;
+                    break;
+                }
+            }
         }
+        surface_client = wl_resource_get_client(send_surf->resource);
 
-        wl_resource_for_each(resource, &grab->touch->focus_resource_list) {
-            if (wl_resource_get_client(resource) != surface_client)
-                continue;
+        resource = wl_resource_find_for_client(&grab->touch->resource_list,
+                                               surface_client);
+        if (resource)
+            wl_touch_send_motion(resource, time, touch_id, sx, sy);
 
+        resource = wl_resource_find_for_client(&grab->touch->focus_resource_list,
+                                               surface_client);
+        if (resource)
             wl_touch_send_motion(resource, time, touch_id, sx, sy);
-        }
     }
 }
 
@@ -614,8 +667,8 @@ touch_grab_frame(struct weston_touch_grab *grab)
 {
     struct seat_ctx *seat = wl_container_of(grab, seat, touch_grab);
     struct surface_ctx *surf_ctx;
-    const struct ivi_controller_interface *interface =
-        seat->input_ctx->ivi_controller_interface;
+    const struct ivi_layout_interface *interface =
+        seat->input_ctx->ivi_layout_interface;
 
     /* For each surface_ctx, check for focus and send */
     wl_list_for_each(surf_ctx, &seat->input_ctx->surface_list, link) {
@@ -631,19 +684,16 @@ touch_grab_frame(struct weston_touch_grab *grab)
 
         surf = interface->surface_get_weston_surface(surf_ctx->layout_surface);
         surface_client = wl_resource_get_client(surf->resource);
-        wl_resource_for_each(resource, &grab->touch->resource_list) {
-            if (wl_resource_get_client(resource) != surface_client)
-                continue;
 
+        resource = wl_resource_find_for_client(&grab->touch->resource_list,
+                                               surface_client);
+        if (resource)
             wl_touch_send_frame(resource);
-        }
-
-        wl_resource_for_each(resource, &grab->touch->focus_resource_list) {
-            if (wl_resource_get_client(resource) != surface_client)
-                continue;
 
+        resource = wl_resource_find_for_client(&grab->touch->focus_resource_list,
+                                               surface_client);
+        if (resource)
             wl_touch_send_frame(resource);
-        }
     }
 }
 
@@ -760,13 +810,16 @@ handle_seat_create(struct wl_listener *listener, void *data)
 }
 
 static void
-handle_surface_destroy(struct ivi_layout_surface *layout_surface, void *data)
+handle_surface_destroy(struct wl_listener *listener, void *data)
 {
-    struct input_context *ctx = data;
+    struct input_context *ctx =
+            wl_container_of(listener, ctx, surface_destroyed);
     struct surface_ctx *surf, *next;
+    struct ivi_layout_surface *layout_surface =
+           (struct ivi_layout_surface *) data;
     int surface_removed = 0;
-    const struct ivi_controller_interface *interface =
-        ctx->ivi_controller_interface;
+    const struct ivi_layout_interface *interface =
+        ctx->ivi_layout_interface;
 
     wl_list_for_each_safe(surf, next, &ctx->surface_list, link) {
         if (surf->layout_surface == layout_surface) {
@@ -790,12 +843,15 @@ handle_surface_destroy(struct ivi_layout_surface *layout_surface, void *data)
 }
 
 static void
-handle_surface_create(struct ivi_layout_surface *layout_surface, void *data)
+handle_surface_create(struct wl_listener *listener, void *data)
 {
-    struct input_context *input_ctx = data;
+    struct input_context *input_ctx =
+            wl_container_of(listener, input_ctx, surface_created);
     struct surface_ctx *ctx;
-    const struct ivi_controller_interface *interface =
-        input_ctx->ivi_controller_interface;
+    struct ivi_layout_surface *layout_surface =
+           (struct ivi_layout_surface *) data;
+    const struct ivi_layout_interface *interface =
+        input_ctx->ivi_layout_interface;
 
     wl_list_for_each(ctx, &input_ctx->surface_list, link) {
         if (ctx->layout_surface == layout_surface) {
@@ -842,8 +898,8 @@ input_set_input_focus(struct wl_client *client,
     struct input_context *ctx = controller->input_context;
     struct surface_ctx *surf, *current_surf = NULL;
     struct weston_seat *seat;
-    const struct ivi_controller_interface *interface =
-	ctx->ivi_controller_interface;
+    const struct ivi_layout_interface *interface =
+	ctx->ivi_layout_interface;
     uint32_t caps;
     struct ivi_layout_surface *current_layout_surface;
 
@@ -924,8 +980,8 @@ input_set_input_acceptance(struct wl_client *client,
     int found_seat = 0;
     int found_weston_seat = 0;
     struct weston_seat *w_seat = NULL;
-    const struct ivi_controller_interface *interface =
-        ctx->ivi_controller_interface;
+    const struct ivi_layout_interface *interface =
+        ctx->ivi_layout_interface;
 
     wl_list_for_each(w_seat, &ctx->compositor->seat_list, link) {
         if(strcmp(seat,w_seat->seat_name) == 0) {
@@ -971,8 +1027,8 @@ bind_ivi_input(struct wl_client *client, void *data,
     struct input_controller *controller;
     struct weston_seat *seat;
     struct surface_ctx *surface_ctx;
-    const struct ivi_controller_interface *interface =
-        ctx->ivi_controller_interface;
+    const struct ivi_layout_interface *interface =
+        ctx->ivi_layout_interface;
     controller = calloc(1, sizeof *controller);
     if (controller == NULL) {
         weston_log("%s: Failed to allocate memory for controller\n",
@@ -1016,7 +1072,7 @@ bind_ivi_input(struct wl_client *client, void *data,
 
 static struct input_context *
 create_input_context(struct weston_compositor *ec,
-                     const struct ivi_controller_interface *interface)
+                     const struct ivi_layout_interface *interface)
 {
     struct input_context *ctx = NULL;
     struct weston_seat *seat;
@@ -1028,14 +1084,16 @@ create_input_context(struct weston_compositor *ec,
     }
 
     ctx->compositor = ec;
-    ctx->ivi_controller_interface = interface;
+    ctx->ivi_layout_interface = interface;
     wl_list_init(&ctx->controller_list);
     wl_list_init(&ctx->surface_list);
 
-    /* Add signal handlers for ivi surfaces. Warning: these functions leak
-     * memory. */
-    interface->add_notification_create_surface(handle_surface_create, ctx);
-    interface->add_notification_remove_surface(handle_surface_destroy, ctx);
+    /* Add signal handlers for ivi surfaces. */
+    ctx->surface_created.notify = handle_surface_create;
+    ctx->surface_destroyed.notify = handle_surface_destroy;
+
+    interface->add_listener_create_surface(&ctx->surface_created);
+    interface->add_listener_remove_surface(&ctx->surface_destroyed);
 
     ctx->seat_create_listener.notify = &handle_seat_create;
     wl_signal_add(&ec->seat_created_signal, &ctx->seat_create_listener);
@@ -1050,7 +1108,7 @@ create_input_context(struct weston_compositor *ec,
 
 WL_EXPORT int
 input_controller_module_init(struct weston_compositor *ec,
-                             const struct ivi_controller_interface *interface,
+                             const struct ivi_layout_interface *interface,
                              size_t interface_version)
 {
     struct input_context *ctx = create_input_context(ec, interface);
