@@ -16,7 +16,6 @@
  * limitations under the License.
  *
  ****************************************************************************/
-#include "ilm_client.h"
 #include "ilm_control.h"
 #include "LMControl.h"
 #include "Expression.h"
@@ -320,39 +319,6 @@ COMMAND("set layer|surface <id> visibility <visibility>")
     }
 }
 
-//=============================================================================
-COMMAND("set layer|surface <id> orientation <orientation>")
-//=============================================================================
-{
-    t_ilm_uint id = input->getUint("id");
-    ilmOrientation orientation = (ilmOrientation)input->getInt("orientation");
-
-    if (input->contains("layer"))
-    {
-        ilmErrorTypes callResult = ilm_layerSetOrientation(id, orientation);
-        if (ILM_SUCCESS != callResult)
-        {
-            cout << "LayerManagerService returned: " << ILM_ERROR_STRING(callResult) << "\n";
-            cout << "Failed to set orientation " << orientation << " for layer with ID " << id << "\n";
-            return;
-        }
-
-        ilm_commitChanges();
-    }
-    else if (input->contains("surface"))
-    {
-        ilmErrorTypes callResult = ilm_surfaceSetOrientation(id, orientation);
-        if (ILM_SUCCESS != callResult)
-        {
-            cout << "LayerManagerService returned: " << ILM_ERROR_STRING(callResult) << "\n";
-            cout << "Failed to set orientation " << orientation << " for surface with ID " << id << "\n";
-            return;
-        }
-
-        ilm_commitChanges();
-    }
-}
-
 //=============================================================================
 COMMAND("set screen|layer <id> render order [<idarray>]")
 //=============================================================================
@@ -621,62 +587,56 @@ COMMAND("create layer <layerid> <width> <height>")
 }
 
 //=============================================================================
-COMMAND("create surface <surfaceid> <nativehandle> <width> <height> <pixelformat>")
+COMMAND("destroy layer <id>")
 //=============================================================================
 {
-    unsigned int surfaceid = input->getUint("surfaceid");
-    unsigned int nativeHandle = input->getUint("nativehandle");
-    unsigned int width = input->getUint("width");
-    unsigned int height = input->getUint("height");
-    e_ilmPixelFormat pixelformat = (e_ilmPixelFormat)input->getUint("pixelformat");
+    unsigned int layerid = input->getUint("id");
 
-    ilmErrorTypes callResult = ilm_surfaceCreate(nativeHandle, width, height, pixelformat, &surfaceid);
+    ilmErrorTypes callResult = ilm_layerRemove(layerid);
     if (ILM_SUCCESS != callResult)
     {
         cout << "LayerManagerService returned: " << ILM_ERROR_STRING(callResult) << "\n";
-        cout << "Failed to create surface with ID " << surfaceid << "\n";
+        cout << "Failed to remove layer with ID " << layerid << "\n";
         return;
     }
-}
-
-//=============================================================================
-COMMAND("destroy layer|surface <id>")
-//=============================================================================
-{
-    if (input->contains("layer"))
-    {
-        unsigned int layerid = input->getUint("id");
-
-        ilmErrorTypes callResult = ilm_layerRemove(layerid);
-        if (ILM_SUCCESS != callResult)
-        {
-            cout << "LayerManagerService returned: " << ILM_ERROR_STRING(callResult) << "\n";
-            cout << "Failed to remove layer with ID " << layerid << "\n";
-            return;
-        }
-    }
-    else if (input->contains("surface"))
-    {
-        unsigned int surfaceid = input->getUint("id");
-
-        ilmErrorTypes callResult = ilm_surfaceRemove(surfaceid);
-        if (ILM_SUCCESS != callResult)
-        {
-            cout << "LayerManagerService returned: " << ILM_ERROR_STRING(callResult) << "\n";
-            cout << "Failed to remove surface with ID " << surfaceid << "\n";
-            return;
-        }
-    }
 
     ilm_commitChanges();
 }
 
 //=============================================================================
-COMMAND("get communicator performance")
+COMMAND("add surface <sid> to layer <lid>")
 //=============================================================================
 {
-    (void) input; //suppress warning: unused parameter
-    getCommunicatorPerformance();
+    t_ilm_uint sid = input->getUint("sid");
+    t_ilm_uint lid = input->getUint("lid");
+
+    ilmErrorTypes callResult = ilm_layerAddSurface(lid, sid);
+    if (ILM_SUCCESS != callResult)
+    {
+        cout << "LayerManagerService returned: " << ILM_ERROR_STRING(callResult) << "\n";
+        cout << "Failed to add surface (" << sid << " ) to layer (" << lid << " ) " << "\n";
+        return;
+    }
+
+    ilm_commitChanges();
+}
+
+//=============================================================================
+COMMAND("remove surface <sid> from layer <lid>")
+//=============================================================================
+{
+    t_ilm_uint sid = input->getUint("sid");
+    t_ilm_uint lid = input->getUint("lid");
+
+    ilmErrorTypes callResult = ilm_layerRemoveSurface(lid, sid);
+    if (ILM_SUCCESS != callResult)
+    {
+        cout << "LayerManagerService returned: " << ILM_ERROR_STRING(callResult) << "\n";
+        cout << "Failed to remove surface (" << sid << " ) from layer (" << lid << " ) " << "\n";
+        return;
+    }
+
+    ilm_commitChanges();
 }
 
 //=============================================================================
@@ -718,28 +678,6 @@ COMMAND("analyze surface <surfaceid>")
     analyzeSurface(targetSurfaceId);
 }
 
-//=============================================================================
-COMMAND("scatter [all]")
-//=============================================================================
-{
-    if (input->contains("all"))
-    {
-        scatterAll();
-    }
-    else
-    {
-        scatter();
-    }
-}
-
-//=============================================================================
-COMMAND("demo [<animation_mode=2>]")
-//=============================================================================
-{
-    t_ilm_uint mode = (t_ilm_uint) input->getUint("animation_mode");
-    demo(mode);
-}
-
 //=============================================================================
 COMMAND("export scene to <filename>")
 //=============================================================================
@@ -757,11 +695,3 @@ COMMAND("export xtext to <filename> <grammar> <url>")
     string url = (string) input->getString("url");
     exportXtext(filename, grammar, url);
 }
-
-//=============================================================================
-COMMAND("import scene from <filename>")
-//=============================================================================
-{
-    string filename = (string) input->getString("filename");
-    importSceneFromFile(filename);
-}
